@@ -5,6 +5,10 @@ import numpy as np
 import tifffile
 from tqdm import trange
 from numba import njit, float32, int32, vectorize
+
+import logging
+dynamics_logger = logging.getLogger(__name__)
+
 from . import utils, metrics
 
 try:
@@ -13,6 +17,7 @@ try:
     from . import resnet_torch
     TORCH_ENABLED = True 
     torch_GPU = torch.device('cuda')
+    torch_CPU = torch.device('cpu')
 except:
     TORCH_ENABLED = False
 
@@ -60,53 +65,44 @@ def _extend_centers(T,y,x,ymed,xmed,Lx, niter):
                                             T[(y+1)*Lx + x-1] + T[(y+1)*Lx + x+1])
     return T
 
-def labels_to_flows(labels, files=None):
-    """ convert labels (list of masks or flows) to flows for training model 
-
-    if files is not None, flows are saved to files to be reused
-
-    Parameters
-    --------------
-
-    labels: list of ND-arrays
-        labels[k] can be 2D or 3D, if [3 x Ly x Lx] then it is assumed that flows were precomputed.
-        Otherwise labels[k][0] or labels[k] (if 2D) is used to create flows and cell probabilities.
-
-    Returns
-    --------------
-
-    flows: list of [4 x Ly x Lx] arrays
-        flows[k][0] is labels[k], flows[k][1] is cell probability, flows[k][2] is Y flow, and flows[k][3] is X flow
+tic=time.time()
 
+def _extend_centers_gpu(neighbors, centers, isneighbor, Ly, Lx, n_iter=200, device=torch.device('cuda')):
+    """ runs diffusion on GPU to generate flows for training images or quality control
+    
+    neighbors is 9 x pixels in masks, 
+    centers are mask centers, 
+    isneighbor is valid neighbor boolean 9 x pixels
+    
     """
+    if device is not None:
+        device = device
+    nimg = neighbors.shape[0] // 9
+    pt = torch.from_numpy(neighbors).to(device)
+    
+    T = torch.zeros((nimg,Ly,Lx), dtype=torch.double, device=device)
+    meds = torch.from_numpy(centers.astype(int)).to(device)
+    isneigh = torch.from_numpy(isneighbor).to(device)
+    for i in range(n_iter):
+        T[:, meds[:,0], meds[:,1]] +=1
+        Tneigh = T[:, pt[:,:,0], pt[:,:,1]]
+        Tneigh *= isneigh
+        T[:, pt[0,:,0], pt[0,:,1]] = Tneigh.mean(axis=1)
+  
+    T = torch.log(1.+ T)
+    # gradient positions
+    grads = T[:, pt[[2,1,4,3],:,0], pt[[2,1,4,3],:,1]]
+    dy = grads[:,0] - grads[:,1]
+    dx = grads[:,2] - grads[:,3]
 
-    nimg = len(labels)
-    if labels[0].ndim < 3:
-        labels = [labels[n][np.newaxis,:,:] for n in range(nimg)]
+    mu_torch = np.stack((dy.cpu().squeeze(), dx.cpu().squeeze()), axis=-2)
+    return mu_torch
 
-    if labels[0].shape[0] == 1 or labels[0].ndim < 3:
-        print('NOTE: computing flows for labels (could be done before to save time)')
-        # compute flows        
-        veci = [masks_to_flows(labels[n][0])[0] for n in trange(nimg)]
-        # concatenate flows with cell probability
-        flows = [np.concatenate((labels[n][[0]], labels[n][[0]]>0.5, veci[n]), axis=0).astype(np.float32)
-                    for n in range(nimg)]
-        if files is not None:
-            for flow, file in zip(flows, files):
-                file_name = os.path.splitext(file)[0]
-                tifffile.imsave(file_name+'_flows.tif', flow)
-    else:
-        print('flows precomputed')
-        flows = [labels[n].astype(np.float32) for n in range(nimg)]
-    return flows
 
-def masks_to_flows(masks):
+def masks_to_flows_gpu(masks, device=None):
     """ convert masks to flows using diffusion from center pixel
 
-    Center of masks where diffusion starts is defined to be the 
-    closest pixel to the median of all pixels that is inside the 
-    mask. Result of diffusion is converted into flows by computing
-    the gradients of the diffusion density map. 
+    Center of masks where diffusion starts is defined using COM
 
     Parameters
     -------------
@@ -126,20 +122,86 @@ def masks_to_flows(masks):
         in which it resides 
 
     """
-    if masks.ndim > 2:
-        Lz, Ly, Lx = masks.shape
-        mu = np.zeros((3, Lz, Ly, Lx), np.float32)
-        for z in range(Lz):
-            mu0 = masks_to_flows(masks[z])[0]
-            mu[[1,2], z] += mu0
-        for y in range(Ly):
-            mu0 = masks_to_flows(masks[:,y])[0]
-            mu[[0,2], :, y] += mu0
-        for x in range(Lx):
-            mu0 = masks_to_flows(masks[:,:,x])[0]
-            mu[[0,1], :, :, x] += mu0
-        return mu, None
+    if device is None:
+        device = torch.device('cuda')
 
+    
+    Ly0,Lx0 = masks.shape
+    Ly, Lx = Ly0+2, Lx0+2
+
+    masks_padded = np.zeros((Ly, Lx), np.int64)
+    masks_padded[1:-1, 1:-1] = masks
+
+    # get mask pixel neighbors
+    y, x = np.nonzero(masks_padded)
+    neighborsY = np.stack((y, y-1, y+1, 
+                           y, y, y-1, 
+                           y-1, y+1, y+1), axis=0)
+    neighborsX = np.stack((x, x, x, 
+                           x-1, x+1, x-1, 
+                           x+1, x-1, x+1), axis=0)
+    neighbors = np.stack((neighborsY, neighborsX), axis=-1)
+
+    # get mask centers
+    centers = np.array(scipy.ndimage.center_of_mass(masks_padded, labels=masks_padded, 
+                                                    index=np.arange(1, masks_padded.max()+1))).astype(int)
+    # (check mask center inside mask)
+    valid = masks_padded[centers[:,0], centers[:,1]] == np.arange(1, masks_padded.max()+1)
+    for i in np.nonzero(~valid)[0]:
+        yi,xi = np.nonzero(masks_padded==(i+1))
+        ymed = np.median(yi)
+        xmed = np.median(xi)
+        imin = np.argmin((xi-xmed)**2 + (yi-ymed)**2)
+        centers[i,0] = yi[imin]
+        centers[i,1] = xi[imin]        
+    
+    # get neighbor validator (not all neighbors are in same mask)
+    neighbor_masks = masks_padded[neighbors[:,:,0], neighbors[:,:,1]]
+    isneighbor = neighbor_masks == neighbor_masks[0]
+
+    slices = scipy.ndimage.find_objects(masks)
+    ext = np.array([[sr.stop - sr.start + 1, sc.stop - sc.start + 1] for sr, sc in slices])
+    n_iter = 2 * (ext.sum(axis=1)).max()
+    # run diffusion
+    mu = _extend_centers_gpu(neighbors, centers, isneighbor, Ly, Lx, 
+                             n_iter=n_iter, device=device)
+
+    # normalize
+    mu /= (1e-20 + (mu**2).sum(axis=0)**0.5)
+
+    # put into original image
+    mu0 = np.zeros((2, Ly0, Lx0))
+    mu0[:, y-1, x-1] = mu
+    mu_c = np.zeros_like(mu0)
+    return mu0, mu_c
+
+def masks_to_flows_cpu(masks, device=None):
+    """ convert masks to flows using diffusion from center pixel
+
+    Center of masks where diffusion starts is defined to be the 
+    closest pixel to the median of all pixels that is inside the 
+    mask. Result of diffusion is converted into flows by computing
+    the gradients of the diffusion density map. 
+
+    Parameters
+    -------------
+
+    masks: int, 2D array
+        labelled masks 0=NO masks; 1,2,...=mask labels
+
+    Returns
+    -------------
+
+    mu: float, 3D array 
+        flows in Y = mu[-2], flows in X = mu[-1].
+        if masks are 3D, flows in Z = mu[0].
+
+    mu_c: float, 2D array
+        for each pixel, the distance to the center of the mask 
+        in which it resides 
+
+    """
+    
     Ly, Lx = masks.shape
     mu = np.zeros((2, Ly, Lx), np.float64)
     mu_c = np.zeros((Ly, Lx), np.float64)
@@ -177,6 +239,103 @@ def masks_to_flows(masks):
 
     return mu, mu_c
 
+def masks_to_flows(masks, use_gpu=False, device=None):
+    """ convert masks to flows using diffusion from center pixel
+
+    Center of masks where diffusion starts is defined to be the 
+    closest pixel to the median of all pixels that is inside the 
+    mask. Result of diffusion is converted into flows by computing
+    the gradients of the diffusion density map. 
+
+    Parameters
+    -------------
+
+    masks: int, 2D or 3D array
+        labelled masks 0=NO masks; 1,2,...=mask labels
+
+    Returns
+    -------------
+
+    mu: float, 3D or 4D array 
+        flows in Y = mu[-2], flows in X = mu[-1].
+        if masks are 3D, flows in Z = mu[0].
+
+    mu_c: float, 2D or 3D array
+        for each pixel, the distance to the center of the mask 
+        in which it resides 
+
+    """
+    if TORCH_ENABLED and use_gpu:
+        if use_gpu and device is None:
+            device = torch_GPU
+        elif device is None:
+            device = torch_CPU
+        masks_to_flows_device = masks_to_flows_gpu 
+    else:
+        masks_to_flows_device = masks_to_flows_cpu
+        
+    if masks.ndim==3:
+        Lz, Ly, Lx = masks.shape
+        mu = np.zeros((3, Lz, Ly, Lx), np.float32)
+        for z in range(Lz):
+            mu0 = masks_to_flows_device(masks[z], device=device)[0]
+            mu[[1,2], z] += mu0
+        for y in range(Ly):
+            mu0 = masks_to_flows_device(masks[:,y], device=device)[0]
+            mu[[0,2], :, y] += mu0
+        for x in range(Lx):
+            mu0 = masks_to_flows_device(masks[:,:,x], device=device)[0]
+            mu[[0,1], :, :, x] += mu0
+        return mu, None
+    elif masks.ndim==2:
+        mu = masks_to_flows_device(masks, device=device)[0]
+        return mu, None
+    else:
+        raise ValueError('masks_to_flows only takes 2D or 3D arrays')
+
+
+
+def labels_to_flows(labels, files=None):
+    """ convert labels (list of masks or flows) to flows for training model 
+
+    if files is not None, flows are saved to files to be reused
+
+    Parameters
+    --------------
+
+    labels: list of ND-arrays
+        labels[k] can be 2D or 3D, if [3 x Ly x Lx] then it is assumed that flows were precomputed.
+        Otherwise labels[k][0] or labels[k] (if 2D) is used to create flows and cell probabilities.
+
+    Returns
+    --------------
+
+    flows: list of [4 x Ly x Lx] arrays
+        flows[k][0] is labels[k], flows[k][1] is cell probability, flows[k][2] is Y flow, and flows[k][3] is X flow
+
+    """
+
+    nimg = len(labels)
+    if labels[0].ndim < 3:
+        labels = [labels[n][np.newaxis,:,:] for n in range(nimg)]
+
+    if labels[0].shape[0] == 1 or labels[0].ndim < 3:
+        dynamics_logger.info('NOTE: computing flows for labels (could be done before to save time)')
+        # compute flows        
+        veci = [masks_to_flows(labels[n][0])[0] for n in trange(nimg)]
+        # concatenate flows with cell probability
+        flows = [np.concatenate((labels[n][[0]], labels[n][[0]]>0.5, veci[n]), axis=0).astype(np.float32)
+                    for n in range(nimg)]
+        if files is not None:
+            for flow, file in zip(flows, files):
+                file_name = os.path.splitext(file)[0]
+                tifffile.imsave(file_name+'_flows.tif', flow)
+    else:
+        dynamics_logger.info('flows precomputed')
+        flows = [labels[n].astype(np.float32) for n in range(nimg)]
+    return flows
+
+
 @njit(['(int16[:,:,:],float32[:], float32[:], float32[:,:])', 
         '(float32[:,:,:],float32[:], float32[:], float32[:,:])'], cache=True)
 def map_coordinates(I, yc, xc, Y):
@@ -211,10 +370,11 @@ def map_coordinates(I, yc, xc, Y):
                       np.float32(I[c, yf1, xf]) * y * (1 - x) +
                       np.float32(I[c, yf1, xf1]) * y * x )
 
-def steps2D_interp(p, dP, niter, use_gpu=False):
+def steps2D_interp(p, dP, niter, use_gpu=False, device=None):
     shape = dP.shape[1:]
     if use_gpu and TORCH_ENABLED:
-        device = torch_GPU
+        if device is None:
+            device = torch_GPU
         pt = torch.from_numpy(p[[1,0]].T).double().to(device)
         pt = pt.unsqueeze(0).unsqueeze(0)
         pt[:,:,:,0] = (pt[:,:,:,0]/(shape[1]-1)) # normalize to between  0 and 1
@@ -320,7 +480,7 @@ def steps2D(p, dP, inds, niter):
             p[1,y,x] = min(shape[1]-1, max(0, p[1,y,x] - dP[1,p0,p1]))
     return p
 
-def follow_flows(dP, niter=200, interp=True, use_gpu=False):
+def follow_flows(dP, niter=200, interp=True, use_gpu=True, device=None):
     """ define pixels and run dynamics to recover masks in 2D
     
     Pixels are meshgrid. Only pixels with non-zero cell-probability
@@ -365,14 +525,18 @@ def follow_flows(dP, niter=200, interp=True, use_gpu=False):
         p = np.array(p).astype(np.float32)
         # run dynamics on subset of pixels
         inds = np.array(np.nonzero(np.abs(dP[0])>1e-3)).astype(np.int32).T
+        if inds.ndim < 2 or inds.shape[0] < 5:
+            dynamics_logger.warning('WARNING: no mask pixels found')
+            return p
         if not interp:
             p = steps2D(p, dP, inds, niter)
         else:
             p[:,inds[:,0],inds[:,1]] = steps2D_interp(p[:,inds[:,0], inds[:,1]], 
-                                                      dP, niter, use_gpu=use_gpu)
+                                                      dP, niter, use_gpu=use_gpu,
+                                                      device=device)
     return p
 
-def remove_bad_flow_masks(masks, flows, threshold=0.4):
+def remove_bad_flow_masks(masks, flows, threshold=0.4, use_gpu=False, device=None):
     """ remove masks which have inconsistent flows 
     
     Uses metrics.flow_error to compute flows from predicted masks 
@@ -401,12 +565,12 @@ def remove_bad_flow_masks(masks, flows, threshold=0.4):
         size [Ly x Lx] or [Lz x Ly x Lx]
     
     """
-    merrors, _ = metrics.flow_error(masks, flows)
+    merrors, _ = metrics.flow_error(masks, flows, use_gpu, device)
     badi = 1+(merrors>threshold).nonzero()[0]
     masks[np.isin(masks, badi)] = 0
     return masks
 
-def get_masks(p, iscell=None, rpad=20, flows=None, threshold=0.4):
+def get_masks(p, iscell=None, rpad=20, flows=None, threshold=0.4, use_gpu=False, device=None):
     """ create masks using pixel convergence after running dynamics
     
     Makes a histogram of final pixel locations p, initializes masks 
@@ -514,7 +678,7 @@ def get_masks(p, iscell=None, rpad=20, flows=None, threshold=0.4):
     for i in range(dims):
         pflows[i] = pflows[i] + rpad
     M0 = M[tuple(pflows)]
-    
+
     # remove big masks
     _,counts = np.unique(M0, return_counts=True)
     big = np.prod(shape0) * 0.4
@@ -523,8 +687,8 @@ def get_masks(p, iscell=None, rpad=20, flows=None, threshold=0.4):
     _,M0 = np.unique(M0, return_inverse=True)
     M0 = np.reshape(M0, shape0)
 
-    if threshold is not None and threshold > 0 and flows is not None:
-        M0 = remove_bad_flow_masks(M0, flows, threshold=threshold)
+    if M0.max()>0 and threshold is not None and threshold > 0 and flows is not None:
+        M0 = remove_bad_flow_masks(M0, flows, threshold=threshold, use_gpu=use_gpu, device=device)
         _,M0 = np.unique(M0, return_inverse=True)
         M0 = np.reshape(M0, shape0).astype(np.int32)
 
