@@ -117,7 +117,7 @@ def run(image=None):
         cp_dir = pathlib.Path.home().joinpath('.cellpose')
         cp_dir.mkdir(exist_ok=True)
         print('downloading logo')
-        utils.download_url_to_file('http://www.cellpose.org/static/images/cellpose_transparent.png', icon_path, progress=True)
+        utils.download_url_to_file('https://www.cellpose.org/static/images/cellpose_transparent.png', icon_path, progress=True)
     if not guip_path.is_file():
         utils.download_url_to_file('https://github.com/MouseLand/cellpose/raw/master/docs/_static/cellpose_gui.png', guip_path, progress=True)
     icon_path = str(icon_path.resolve())
@@ -1334,7 +1334,7 @@ class MainW(QtGui.QMainWindow):
             #    masks = masks[0][np.newaxis,:,:]
             #    flows = flows[0]
             self.flows[0] = flows[0].copy()
-            self.flows[1] = (np.clip(utils.normalize99(flows[2]),0,1) * 255).astype(np.uint8)
+            self.flows[1] = (np.clip(utils.normalize99(flows[2].copy()),0,1) * 255).astype(np.uint8)
             if not do_3D:
                 masks = masks[np.newaxis,...]
                 self.flows[0] = transforms.resize_image(self.flows[0], masks.shape[-2], masks.shape[-1],
