@@ -3,6 +3,7 @@ from natsort import natsorted
 import numpy as np
 import cv2
 import tifffile
+import logging
 
 from . import utils, plot, transforms
 
@@ -17,15 +18,13 @@ try:
     MATPLOTLIB = True
 except:
     MATPLOTLIB = False
-    print('matplotlib not installed')
-
+    
 try:
     from google.cloud import storage
     SERVER_UPLOAD = True
 except:
     SERVER_UPLOAD = False
 
-
 def outlines_to_text(base, outlines):
     with open(base + '_cp_outlines.txt', 'w') as f:
         for o in outlines:
@@ -227,7 +226,8 @@ def masks_flows_to_seg(images, masks, flows, diams, file_names, channels=None):
         if images.shape[0]<8:
             np.transpose(images, (1,2,0))
         np.save(base+ '_seg.npy',
-                    {'outlines': outlines.astype(np.uint16) if outlines.max()<2**16-1 else outlines.astype(np.uint32),
+                    {'img': images,
+                        'outlines': outlines.astype(np.uint16) if outlines.max()<2**16-1 else outlines.astype(np.uint32),
                      'masks': masks.astype(np.uint16) if masks.max()<2**16-1 else masks.astype(np.uint32),
                      'chan_choose': channels,
                      'ismanual': np.zeros(masks.max(), np.bool),
